function [Etechtab,Ntechtab,montab,alltab,histhp,histbp,varnamz,hornmz] = getvardecomp(a,b,lam,simdataout,vdout,erzout,azeroout,a0betazout)
%
%This routine returns the fraction of total variance due to three shocks, for each of the 11 variables in varnamz.
%simdataout, vdout, erzout, azeroout, a0betazout are produced by mkimplrnew.m
%a, b, lam are user-supplied, as discussed below
%
%Etechtab - 11 by 7 matrix of fractions of variance due to the embodied technology shock
%           7 columns of Etechtab are as indicated in hornmz: 
%             column 1 - 1 quarter ahead forecast error variance
%             column 2 - 4 quarter ahead forecast error variance
%             column 3 - 8 quarter ahead forecast error variance
%             column 4 - 12 quarter ahead forecast error variance
%             column 5 - 30 quarter ahead forecast error variance (or, whatever is the second dimension of vdout)
%             column 6 - fraction of variance in hp filtered data with parameter 1600
%             column 7 - fraction of variance in bandpass filtered data containing components with period greater than a and less than b
%Ntechtab - same as Etechtab, but applied to neutral technology shock
%montab - same as Etechtab, but applied to monetary policy shock
%alltab - sum of Etechtab, Ntechtab, montab
%
%histhp - computes variance decompositions in a manner analogous to the
%         output generated by gethistorical.m. Simulate one realization of data in
%         response to historical realization of fitted disturbances. HP filter the result. Simulate
%         another realization of data in response to only the historical realization of shock x.
%         HP filter the result. The ratio of the variance hp filtered data under one shock to the hp 
%         filtered data under two shocks is the variance decomposition. histhp has
%         four columns. First column corresponds to x=embodied technology shock;
%         second - x=neutral technology shock; third - x=monetary policy shock;
%         fourth - sum of fractions for the three shocks.
%
%histbp - same as histhp, except done for band pass filter, allowing
%         components with period greater than a and less than b to pass.


%get the variance decompositions associated with the band pass and hp filters. this is done using a frequency domain algorithm
[hp,bp,varnamz] = spectdecomp(a,b,lam,erzout,azeroout,a0betazout);
hp=100*hp;
bp=100*bp;

nsteps=size(vdout,2);
hornmz  = char('1','4','8','12',num2str(nsteps),'HP','BP');

hp1    =   simdataout(:,:,1)-HPF(simdataout(:,:,1),1600);
hp5    =   simdataout(:,:,5)-HPF(simdataout(:,:,5),1600);
bp1    =   bpassgen(simdataout(:,:,1),8,32);
bp5    =   bpassgen(simdataout(:,:,5),8,32);
bp1lr   =   bpassgen(simdataout(:,:,1),32,80);
bp5lr   =   bpassgen(simdataout(:,:,5),32,80);
bp1sr   =   bpassgen(simdataout(:,:,1),2,8);
bp5sr   =   bpassgen(simdataout(:,:,5),2,8);
hpEtech  =   100*var(hp1)./var(hp5);
bpEtech  =   100*var(bp1)./var(bp5);
bpEtechLR   =   100*var(bp1lr)./var(bp5lr);
bpEtechSR   =   100*var(bp1sr)./var(bp5sr);

Etechtab    =   [100*vdout(:,1,1),100*vdout(:,4,1),100*vdout(:,8,1),100*vdout(:,12,1),100*vdout(:,nsteps,1),hp(:,1),bp(:,1)];

hp2    =   simdataout(:,:,2)-HPF(simdataout(:,:,2),1600);
bp2    =   bpassgen(simdataout(:,:,2),8,32);
bp2lr  =   bpassgen(simdataout(:,:,2),32,80);
bp2sr  =   bpassgen(simdataout(:,:,2),2,8);
bpNtechLR   =   100*var(bp2lr)./var(bp5lr);
bpNtechSR   =   100*var(bp2sr)./var(bp5sr);
bpNtech  =   100*var(bp2)./var(bp5);
hpNtech  =   100*var(hp2)./var(hp5);

Ntechtab    =   [100*vdout(:,1,2),100*vdout(:,4,2),100*vdout(:,8,2),100*vdout(:,12,2),100*vdout(:,nsteps,2),hp(:,2),bp(:,2)];

hp3    =   simdataout(:,:,3)-HPF(simdataout(:,:,3),1600);
bp3    =   bpassgen(simdataout(:,:,3),8,32);
bp3lr  =   bpassgen(simdataout(:,:,3),32,80);
bp3sr  =   bpassgen(simdataout(:,:,3),2,8);
bpmon  =   100*var(bp3)./var(bp5);
bpmonLR=   100*var(bp3lr)./var(bp5lr);
bpmonSR=   100*var(bp3sr)./var(bp5sr);
hpmon  =   100*var(hp3)./var(hp5);

montab    =   [100*vdout(:,1,3),100*vdout(:,4,3),100*vdout(:,8,3),100*vdout(:,12,3),100*vdout(:,nsteps,3),hp(:,3),bp(:,3)];

hp4    =   simdataout(:,:,4)-HPF(simdataout(:,:,4),1600);%when the third dimension is 4, this means simulation based on the three identified shocks
bp4    =   bpassgen(simdataout(:,:,4),8,32);
bp4lr  =   bpassgen(simdataout(:,:,4),32,80);
bp4sr  =   bpassgen(simdataout(:,:,4),2,8);
bpall  =   100*var(bp4)./var(bp5);
bpallLR=   100*var(bp4lr)./var(bp5lr);
bpallSR=   100*var(bp4sr)./var(bp5sr);
hpall  =   100*var(hp4)./var(hp5);

alltab    =   [100*([vdout(:,1,1)+vdout(:,1,2)+vdout(:,1,3)]), ...
        100*([vdout(:,4,1)+vdout(:,4,2)+vdout(:,4,3)]), ...
        100*([vdout(:,8,1)+vdout(:,8,2)+vdout(:,8,3)]), ...
        100*([vdout(:,12,1)+vdout(:,12,2)+vdout(:,12,3)]), ...
        100*([vdout(:,nsteps,1)+vdout(:,nsteps,2)+vdout(:,nsteps,3)]),hp(:,4),bp(:,4)];%the fourth dimension of vdout is the velocity shock

histhp=[hpEtech' hpNtech' hpmon' hpall'];
histbp=[bpEtech' bpNtech' bpmon' bpall'];
    
LR      =   [bpEtechLR',bpNtechLR',bpmonLR',bpallLR'];
SR      =   [bpEtechSR',bpNtechSR',bpmonSR',bpallSR'];
