function mat2textable (xx, f, heads, caption, label)
% mat2textable (var, f, heads, caption, label)
%
% Formats the values in 'var' to be included in a LaTex2e 
% source file. The values are formated as a Latex2e Table. 
%
% Input Arguments: 
%  var............Matrix containing real values
%                 Cannot handle text strings. 
%  f..............Optional. File stream where the Table will be written
%                 Default value of f = 1
%  heads..........Optional. Headings for the table columns specified as 
%                 cell array. Default is no heading
%  caption........Optional. The caption for the table. Default is
%                 'Generated from Matlab'
%  label..........Optional. The reference label for the table. Default is
%                 'tab:matlab'
%
%  Usage: 
%      If A is a 3x10 matrix containing the temperature, pressure
%      and flow values from an experiment or simulation then, 
% 
%        f = fopen('Table1.tex', 'wt'); 
%        mat2textable (A, f, {'Temp', 'Pres', 'Flow'}, 'Expt 1', 'tab:expt1');
%        fclose(f); 
%
%      will generate a Latex2e table with the given headings and
%      the values of A. The table will be centered with a caption
%      'Expt 1' and a reference label 'tab:expt1'
%
%      You can then include 'Table1.tex' in your main Latex source file
%      \input{Table1}
%
%
%  Notes: 
%   1. If you are using reserved characters (e.g. #, \, % etc) in the headings, 
%      caption or label, be sure to follow Latex2e instructions. 
%   2. If you do not like the table format provide by this function you will need
%      knowledge about Latex2e to modify this function
%   3. This requires Matlab version 5.3 or higher.
%
% $Revision$
% author: d. mylaraswamy (dinkar.a.mylaraswamy@honeywell.com)
% Created: 8/21/00
% 
%


%%%% This simple m-file, does no processing, but merely inserts the
%% appropriate LATEX commands for generating a Table environment and
%% uses fprintf to write the numerical values

%% print to stdout ?
if nargin < 2, 
   f = 1; 
end; 

%% set the default values for the caption and the label 
if nargin < 3, 
   caption = 'Generated by Matlab'; 
   label = 'tab:matlab'; 
end; 
if nargin < 4,
   label = 'tab:matlab'; 
end; 

%% cannot handle non-double arrays
if ~isa(xx, 'double')
   error(sprintf('%s must be a double array', inputname(1))); 
end; 

%% Get the size of the input matrix 
[nRows, nCols] = size(xx); 

%% did the user do a goofy thing ??
if nRows <= 0 | nCols <= 0 
   error(sprintf('%s empty array', inputname(1))); 
end;

%% make sure that the number of header and the number of columns of xx
%% are consistent
if nargin > 2, 
   if length(heads) ~= nCols, 
      error(sprintf('Number of columns of %s did not match the header', inputname(1)));
   end;
end;


%% A default format for the table is specified as
%% 2 vertical lines at the left and right
%% and a vertical line in-between the columns. 
hh = '||'; 
bb = ' '; 
%% repeat the column format over all the columns
for i = 1:nCols, 
   hh = [hh 'l|']; 
end; 
%% Latex2e uses a '&' as a delimited between columns
for i = 2:nCols, 
   bb = [bb '& ']; 
end; 

%% the final column is treated specially ....
hh = [hh '|']; 
bb = [bb '\\']; 

%% print the preamble of the table 
%% see Leslie Lamport's LATEX book for details. 
%% open the table environment as a floating body 
fprintf(f, '\\begin{table}[htbp] \n'); 
fprintf(f, '\\begin{center} \n'); 
fprintf(f, '\\begin{tabular}{%s} \n', hh); 
fprintf(f, '\\hline \n'); 

%% Is there a column heading that needs to be printed ?
if nargin > 2, 
   fprintf(f, '%s\n', bb); 
   fprintf(f, '{\\bf %s}', char(heads{1}) ); 
   for i = 2:nCols, 
      fprintf(f, ' &{\\bf %s}', char(heads{i}) ); 
   end; 
   fprintf(f, '\\\\ \n'); 
   fprintf(f, '%s\n', bb); 
	fprintf(f, '\\hline \n');
end;

%% Now print the values of the matrix ... 
for k = 1:nRows, 
   fprintf(f, '$%f$', xx(k,1)); 
   for i = 2:nCols, 
      fprintf(f, ' &$%f$', xx(k,i) ); 
   end; 
   fprintf(f, '\\\\ \n');
end; 

%% print the footer of the table environment
fprintf(f, '%s\n', bb); 
fprintf(f, '\\hline \n'); 
fprintf(f, '\\end{tabular} \n');
%% include the user-defined or default caption
fprintf(f, '\\caption{%s} \n', caption); 
%% include the user-defined or default label 
fprintf(f, '\\label{%s} \n', label); 
fprintf(f, '\\end{center} \n'); 
%% close the table environment and return 
fprintf(f, '\\end{table} \n'); 



